// ... (Keep previous imports and constants)

// Modified Lore Handling Functions
function parseLoreEntry(entry) {
    try {
        const data = JSON.parse(entry);
        return { 
            pos: data.pos, 
            name: data.name || `Location at ${data.pos}`,
            type: data.type || "loader"
        };
    } catch {
        return {
            pos: entry,
            name: `Location at ${entry}`,
            type: "loader"
        };
    }
}

function createLoreEntry(pos, name, type = "loader") {
    return JSON.stringify({ pos, name, type });
}

// Updated Add Loader Position with Naming
function addLoaderPositionToStick(player, block) {
    const inv = getPlayerInventory(player);
    const item = inv.getItem(player.selectedSlotIndex);
    const lore = item?.getLore() || [];

    const position = `${block.x} ${block.y} ${block.z}`;
    
    // Check if already exists
    const exists = lore.some(entry => {
        const data = parseLoreEntry(entry);
        return data.pos === position;
    });
    
    if (exists) {
        sendMessage(player, "This Cloud Loader is already linked!", "c");
        playErrorEffect(player);
        return;
    }

    new ModalFormData()
        .title("Name this Link")
        .textField("Enter a display name:", `My ${block.typeId === "minecraft:chest" ? "Chest" : "Loader"}`)
        .show(player).then(res => {
            if (res.canceled) return;
            
            const name = res.formValues[0]?.trim() || 
                `Linked ${block.typeId === "minecraft:chest" ? "Chest" : "Loader"} at ${position}`;
            
            lore.push(createLoreEntry(
                position, 
                name,
                block.typeId === "minecraft:chest" ? "chest" : "loader"
            ));
            
            item.setLore(lore);
            inv.setItem(player.selectedSlotIndex, item);
            
            sendMessage(player, `Successfully linked: §b${name}`);
            playSuccessEffect(player);
        });
}

// Enhanced Storage Selection Menu
function openStorageFromStick(player) {
    const inv = getPlayerInventory(player);
    const item = inv.getItem(player.selectedSlotIndex);
    const lore = item?.getLore();

    if (!lore?.length) {
        sendMessage(player, "Storage Stick not linked!", "c");
        playErrorEffect(player);
        return;
    }

    const form = new ActionFormData()
        .title("Linked Storage Locations")
        .body("Select a location to access:");

    lore.forEach(entry => {
        const data = parseLoreEntry(entry);
        form.button(data.name, ICON_PATHS[data.type === "chest" ? "hotbar" : "lena"]);
    });

    form.show(player).then(res => {
        if (res.canceled || res.selection === undefined) return;
        
        const selectedEntry = parseLoreEntry(lore[res.selection]);
        const [x, y, z] = selectedEntry.pos.split(" ").map(Number);
        const block = world.getDimension("overworld").getBlock({ x, y, z });

        sendMessage(player, `Selected: §b${selectedEntry.name}§r\nPosition: §a${x} ${y} ${z}`);

        if (block?.typeId === "um:storge") {
            openLoaderMenu(player, block, false);
        } else if (block?.typeId === "minecraft:chest") {
            showChestStatus(player, [selectedEntry.pos]);
        } else {
            sendMessage(player, "Linked block missing!", "c");
            playErrorEffect(player);
        }
    });
}

// Updated Chest Linking Function
function addChestPositionToStick(player, block) {
    const inv = getPlayerInventory(player);
    const item = inv.getItem(player.selectedSlotIndex);
    const lore = item?.getLore() || [];

    if (lore.length >= 5) {
        sendMessage(player, "Max 5 connections per stick!", "c");
        playErrorEffect(player);
        return;
    }

    addLoaderPositionToStick(player, block); // Reuse the naming function
}

// Enhanced Loader Menu
function openLoaderMenu(player, block, showRemove) {
    const positionKey = `cloudstorage:loader_${block.x},${block.y},${block.z}`;
    const chestData = world.getDynamicProperty(positionKey);

    if (!chestData) {
        sendMessage(player, "No connected chests!", "c");
        playErrorEffect(player);
        return;
    }

    const parsedData = JSON.parse(chestData).map(parseLoreEntry);
    const connectionCount = parsedData.length;

    const form = new ActionFormData()
        .title("Cloud Loader Interface")
        .body(`Connected to ${connectionCount} chests`)
        .button("Withdraw Items", ICON_PATHS.lena)
        .button("Deposit Items", ICON_PATHS.add)
        .button("Chest Network Status", ICON_PATHS.hotbar);

    if (showRemove) form.button("Remove Loader", ICON_PATHS.remove);

    form.show(player).then(res => {
        if (res.canceled) return;
        
        switch(res.selection) {
            case 0: 
                showChestItems(player, parsedData);
                break;
            case 1: 
                sendItemsToChests(player, parsedData);
                break;
            case 2: 
                showChestStatus(player, parsedData);
                break;
            case 3: 
                removeLoader(block, player); 
                break;
        }
    });
}

// Updated Status Display
function showChestStatus(player, lore) {
    let status = "§6Connected Storage:\n§r";
    lore.forEach((entry, i) => {
        const [x, y, z] = entry.pos.split(" ").map(Number);
        const block = world.getDimension("overworld").getBlock({ x, y, z });
        
        status += `\n§l${i+1}. §r§b${entry.name}§r\n`;
        status += `Position: §a${x} ${y} ${z}§r\n`;
        
        if (block?.typeId === "minecraft:chest") {
            const chest = block.getComponent("inventory").container;
            const used = Array.from({ length: chest.size }, (_, i) => chest.getItem(i)).filter(Boolean).length;
            status += `Status: §2${used}/${chest.size} slots used§r\n`;
        } else {
            status += "Status: §cMissing!§r\n";
        }
    });
    
    new MessageFormData()
        .title("Storage Network Status")
        .body(status)
        .button1("OK")
        .show(player);
}

// ... (Keep remaining functions with updated lore handling where needed)